from Script.Layer.Layer import Layer
from Script.assist.Array import Array
from Script.Player.Knight import Knight
class CollisionLayer(Layer):
    def __init__(self,cols,rows):
        Layer.__init__(self,cols,rows)
    def UpdateCollision(self,array):
        for i in range(self.cols):
            for j in range(self.rows):
                if array[i,j] == 1:
                    Knight.GetInstance().collider[i,j] = 1
                elif array[i,j] == 2:
                    Knight.GetInstance().collider[i,j] = 2
                elif array[i,j] == 3:
                    Knight.GetInstance().collider[i,j] = 3
                elif array[i,j] == 4:
                    Knight.GetInstance().collider[i,j] = 4
                elif array[i,j] == 5:
                    Knight.GetInstance().collider[i,j] = 5
                elif array[i,j] == 6:
                    Knight.GetInstance().collider[i,j] = 6
                elif array[i,j] == 7:
                    Knight.GetInstance().collider[i,j] = 7
                elif array[i,j] == 8:
                    Knight.GetInstance().collider[i,j] = 8
                elif array[i,j] == 9:
                    Knight.GetInstance().collider[i,j] = 9
                elif array[i,j] == 10:
                    Knight.GetInstance().collider[i,j] = 10
                elif array[i,j] == 11:
                    Knight.GetInstance().collider[i,j] = 11
        print('更新碰撞完成，碰撞矩阵为：')
        for x in Knight.GetInstance().collider.array:
            print(x)
        print('----------end----------')


from Script.Layer.Layer import Layer
from Script.boulder.boulder import boulder
from Script.assist.Array import Array
class BoulderLayer(Layer):
    def __init__(self,cols,rows,array,blank,collider):
        Layer.__init__(self,cols,rows)
        self.array=Array()
        self.array.Set(array)
        self.CreateLayer(blank,array,collider)
    def CreateLayer(self,blank,array,collider):
       temp=boulder(blank,array,collider)
                    







from concurrent.futures import ThreadPoolExecutor
import threading, time


class WorkThread(threading.Thread):
    def __init__(self):
        threading.Thread.__init__(self)

    def run(self) -> None:
        print('start run task')
        time.sleep(2)
        print('task finish\n')
        return '线程类执行完成'

def test(thread_name):
    print(thread_name, threading.current_thread().name)
    time.sleep(5)
    print('任务完成\n')
    return '线程方法执行完成'

if __name__ == '__main__':
    thread_pool = ThreadPoolExecutor(5)
    futures = []
    for i in range(7):
        thraed = WorkThread()
        # sumit(方法名，参数)
        future1 = thread_pool.submit(thraed.run)
        future2 = thread_pool.submit(test, i)
        futures.append(future1)
        futures.append(future2)

    def get_call_back(future):
        # 监听任务执行结果，当前线程一直阻塞知道有结果，但是不阻塞主线程
        print(future.result())

    for future in futures:
        #添加监听
        future.add_done_callback(get_call_back)

    print('main thread')



from Script.Layer.Layer import Layer
from Script.mill.mill import mill
from Script.assist.Array import Array
class MillLayer(Layer):
    def __init__(self,cols,rows,array,blank,collider):
        Layer.__init__(self,cols,rows)
        self.array=array
        self.element=mill(self.array)
        self.CreateLayer(blank,collider)
    def CreateLayer(self,blank,collider):
        for i in range(self.cols):
            for j in range(self.rows):
                if self.array[i,j]!=0:
                    collider[i,j] = 7
                    blank.millList.append(self.element)



import time as pt
class Time(object):
    time=0
    deltaTime=0
    fps = 1/60
    def __init__(self):
        self.oldTime=0
        self.newTime=0
    def CalculationDeltaTime(self):
        self.oldTime=self.newTime
        self.newTime=pt.time()
        temp=self.newTime-self.oldTime
        if temp <Time.fps:
            pt.sleep(Time.fps-temp)
            self.newTime=pt.time()
        Time.deltaTime=self.newTime-self.oldTime
        if Time.deltaTime>100:
            Time.deltaTime=0
        Time.time+=Time.deltaTime
    @staticmethod
    def SetFPS(fps: int):
        if fps>0:
            Time.fps=1/fps


import random
import pygame
import sys
import os
from pygame.locals import *
from Resources.sprites.sprite import Sprite
class bridge(Sprite):
    def __init__(self):
        Sprite.__init__(self)
        n=0
        for x in range(2):
            self.element.append(pygame.image.load(Get('bridge_' + str(n) + '.png')))
            n+=1
    def GetElement(self,x):
        return self.element[x]

def Get(name):
    return os.path.dirname(os.path.realpath(__file__))+"\\"+name


from Script.MyScreen import MyScreen
from Script.AllEvent import AllEvent
from Script.Player.Knight import Knight
from Resources.sprites.boulder.boulder import boulder as b
import copy
import time
class boulder:
    def __init__(self,blank,array,collider):
        self.e = b()
        self.blank = blank
        self.collider = collider
        self.active=False
        self.array = array
        self.active = False
        for i in range(MyScreen.cols):
            for j in range(MyScreen.rows):
                if self.array[i,j] == 1:
                    self.aa=(i,j)
                    collider[i,j] = 1
                if self.array[i,j] == 2:
                    self.bb = (i,j)
        self.posX,self.posY = self.GetPos()
        self.AddList()
        AllEvent.AddEventA(self.SetActive)
    def AddList(self):
        self.blank.boulderList.append(self)
        pass
    def GetImage(self):
        return self.e.GetElement()
    def GetPos(self):
        a=int((1000-MyScreen.rows*65)/2+self.aa[1]*65)
        b=int((800-MyScreen.cols*65)/2+self.aa[0]*65)
        return (a,b)
    def GetBPos(self):
        return (self.posX,self.posY)
    def SetActive(self):
        self.active = not self.active
        self.Move()
    def Move(self):
        a = self.aa[0] - self.bb[0]
        b = self.aa[1] - self.bb[1]
        if self.active:
            if b>0:
                for x in range(0,b):
                    for i in range(65):
                        time.sleep(0.01)
                        self.posX-=1
            else:
                for x in range(0,b*-1):
                    for i in range(65):
                        time.sleep(0.01)
                        self.posX+=1
            if a>0:
                for x in range(0,a):
                    for i in range(65):
                        time.sleep(0.01)
                        self.posY-=1
            else:
                for x in range(0,a*-1):
                    for i in range(65):
                        time.sleep(0.01)
                        self.posY+=1
            Knight.GetInstance().collider[self.aa[0],self.aa[1]] = 0
            Knight.GetInstance().collider[self.bb[0],self.bb[1]] = 1
        else:
            if b>0:
                for x in range(0,b):
                    for i in range(65):
                        time.sleep(0.01)
                        self.posX+=1
            else:
                for x in range(0,b*-1):
                    for i in range(65):
                        time.sleep(0.01)
                        self.posX-=1
            if a>0:
                for x in range(0,a):
                    for i in range(65):
                        time.sleep(0.01)
                        self.posY+=1
            else:
                for x in range(0,a*-1):
                    for i in range(65):
                        time.sleep(0.01)
                        self.posY-=1
            Knight.GetInstance().collider[self.aa[0],self.aa[1]] = 1
            Knight.GetInstance().collider[self.bb[0],self.bb[1]] = 0
        print('.OK')



from Script.Layer.Layer import Layer
from Resources.sprites.background.BackGround import BackGround
from Script.assist.Array import Array
class BackGroundLayer(Layer):
    def __init__(self):
        Layer.__init__(self,0,0)
        self.element=BackGround()
    def GetBackGround(self):
        return self.element.GetEement()




from Script.Layer.Layer import Layer
from Script.Player.Knight import Knight
from Script.assist.Array import Array
class PlayerLayer(Layer):
    def __init__(self,cols,rows,array):
        Layer.__init__(self,cols,rows)
        self.array=Array()
        self.array.Set(array)
        self.element = Knight(array)
        self.CreateLayer()
    def CreateLayer(self):
        pass

    def GetPlayer(self):
        return self.element.GetKnightAnim()
    def GetPlayerPos(self):
        return (self.element.x,self.element.y)



from Script.mcgame.MCUnittest import *
import Script.mcgame.MCGameUnittest as mc
import Maps.MapManager
import _thread
import time
from enum import Enum
print('欢迎使用魔扣少儿编程')
print('您可以尝试输入  MCPython.help()获取帮助信息！')
#print('此版本还未添加打包序列')
print('----------------------------------------------------------------')

def help():
    print('输入  MCPython.LoadMap(关卡)来加载关卡')
    print('例如  MCPython.LoadMap(1)来加载关卡')
    print('')
    print('加载关卡请放在最前面，不然骑士无法知道怎么移动')
    print('')
    print('输入  MCPython.moveFoward()来控制人物移动')
    print('输入  MCPython.turnLeft()来控制人物左转')
    print('输入  MCPython.turnRight()来控制人物右转')
    print('输入  MCPython.pickUp()来控制人物进行操作')
    print('')
    print('输入  MCPython.start()来开始游戏，请务必放在最后')
    print('----------------------------------------------------------------')


action = unittest.TestSuite()
updatescreen=unittest.TestSuite()

def chooseMap(id):
    temp = None
    if id == 1:
        temp = Maps.MapManager.LoadLevel01()
    elif id == 2:
        temp = Maps.MapManager.LoadLevel02()
    elif id == 3:
        temp = Maps.MapManager.LoadLevel03()
    elif id == 4:
        temp = Maps.MapManager.LoadLevel04()
    elif id == 5:
        temp = Maps.MapManager.LoadLevel05()
    elif id == 6:
        temp = Maps.MapManager.LoadLevel06()
    elif id == 7:
        temp = Maps.MapManager.LoadLevel07()
    elif id == 8:
        temp = Maps.MapManager.LoadLevel08()
    elif id == 9:
        temp = Maps.MapManager.LoadLevel09()
    elif id == 10:
        temp = Maps.MapManager.LoadLevel10()
    mc.SetMapName(temp)
def actionimplement():
    time.sleep(1)
    unittest.TextTestRunner().run(action)

def moveForward():
    action.addTest(myaction('test_forword_run'))
    print('forword')

def pickUp():
    action.addTest(myaction('test_pickup_run'))
    print('pickUp')

def turnLeft():
    action.addTest(myaction('test_left_run'))
    print('left')

def turnRight():
    action.addTest(myaction('test_right_run'))
    print('right')
def start(): 
    action.addTest(myaction('test_start_run'))
    updatescreen.addTest(myscreen('test_updateScreen_run'))
    updatescreen.addTest(myscreen('test_start_run'))
    _thread.start_new_thread(actionimplement,())
    unittest.TextTestRunner().run(updatescreen)


class AllEvent:
    eventA=[]
    eventB=[]
    eventC=[]
    def RunA():
        if AllEvent.eventA!=[]:
            for x in AllEvent.eventA:
                x()
    def RunC():
        if AllEvent.eventC!=[]:
            for x in AllEvent.eventC:
                x()
    def RunB():
        if AllEvent.eventB!=[]:
            for x in AllEvent.eventB:
                x()
    def AddEventA(event):
        AllEvent.eventA.append(event)
    def AddEventB(event):
        AllEvent.eventB.append(event)
    def AddEventB(event):
        AllEvent.eventB.append(event)





from Script.Layer.Layer import Layer
from Script.equipage.equipage import equ
from Script.assist.Array import Array
class EquipageLayer(Layer):
    def __init__(self,cols,rows,array,blank,collider):
        Layer.__init__(self,cols,rows)
        self.array=array
        self.CreateLayer(blank,collider)
    def CreateLayer(self,blank,collider):
        for i in range(self.cols):
            for j in range(self.rows):
                if self.array[i,j]!=0:
                    if self.array[i,j]==1:
                        blank.equList.append(equ(self.array,'sword'))
                        self.array[i,j] = 0
                        collider[j,i]=3
                    elif self.array[i,j]==2:
                        blank.equList.append(equ(self.array,'shield'))
                        self.array[i,j] = 0
                        collider[i,j]=4
                    elif self.array[i,j]==3:
                        blank.equList.append(equ(self.array,'helmet'))
                        self.array[i,j] = 0
                        collider[i,j]=5





import random
import pygame
import sys
import os
from pygame.locals import *
from Resources.sprites.sprite import Sprite
class Stump(Sprite):
    def __init__(self):
        Sprite.__init__(self)
        n=0
        self.element.append(pygame.image.load(Get('stump_' + str(n) + '.png')))
    def GetElement(self):
        return self.element[0]

def Get(name):
    return os.path.dirname(os.path.realpath(__file__))+"\\"+name



from Resources.sprites.mill.mill import mill as ml
from Script.assist.Array import Array
import pygame
from Script.MyScreen import MyScreen
class mill:
    def __init__(self,array):
        self.cols=array.cols
        self.rows=array.rows
        self.array=array
        self.n=-1
        temp=self.GetPos()
        self.x=temp[0]
        self.y=temp[1]
        self.posY=temp[3]
        self.posX=temp[2]
        self.equn=0
        self.m=ml()
        self.a = self.m.GetElement(0)
        self.b = self.m.GetElement(1)
        self.c = self.b
        self.rect = self.b.get_rect()
        self.rect = self.rect.move((MyScreen.Width - self.rect.width) / 2,(MyScreen.Height - self.rect.height) / 2)
        self.angle = 0
        self.isR = False
        pass
    def SetRotate(self):
        self.isR = not self.isR
    def GetPos(self):
        for i in range(0,self.cols):  
             for j in range(self.rows):
                a=int((1000-self.rows*65)/2+j*65)
                b=int((800-self.cols*65)/2+i*65)
                if self.array[i,j]:
                    return (a,b,i,j)
    def Rotate(self,x,y):
        if self.isR:
            self.b = pygame.transform.rotate(self.c, self.angle)
            self.angle += 2
            if self.angle > 360:
                self.angle -= 360
        newR = self.b.get_rect(center=self.rect.center)
        newR = newR.move(x,y)
        return (self.b,newR)
        pass




__all__=['BackGroundLayer',
         'BlankLayer',
         'CastleLayer',
         'CollisionLayer',
         'GrassLayer',
         'Layer',
         'MixedLayer',
         'PineLayer',
         'PlayerLayer',
         'SoundsLayer',
         'StoneLayer',
         'StumpLayer',
         'WaterLayer',
         'EquipageLayer',
         'BridgeLayer',
         'FloversLayer',
         'SpikesLayer',
         'BoulderLayer'
         'MillLayer']


from Resources.sprites.equipage.equipage import equipage
from Script.assist.Array import Array
from Script.Layer.MixedLayer import *
import os
import pygame
from pygame.locals import *
import Script.assist.ThreadPool
import time
import _thread
import threading
from Script.MyScreen import MyScreen
from enum import Enum
class EquipageAnimation(Enum):
    wolf = 'wolf'
    shield = 'shield'
    helmet = 'helmet'
    sword = 'sword'
    boom = 'boom'
class equ:
    def __init__(self,array,str):
        self.cols=array.cols
        self.rows=array.rows
        self.array=array
        self.n=-1
        temp=self.GetPos()
        self.x=temp[0]
        self.y=temp[1]
        self.posY=temp[3]
        self.posX=temp[2]
        self.equn=0
        self.e=equipage()
        if str == 'sword':
            self.equAnim=self.e.GetEement(EquipageAnimation.sword.value)
            self.equn=3
        elif str == 'shield':
            self.equAnim=self.e.GetEement(EquipageAnimation.shield.value)
            self.equn=4
        elif str == 'helmet':
            self.equAnim=self.e.GetEement(EquipageAnimation.helmet.value)
            self.equn=5
        pass
    def GetPos(self):
        for i in range(0,self.cols):  
             for j in range(self.rows):
                a=int((1000-self.rows*65)/2+j*65+10)
                b=int((800-self.cols*65)/2+i*65)
                if self.array[i,j]:
                    #print(self.array[i,j])
                    #print('abij:'+str((a,b,i,j)))
                    return (a,b,i,j)
    def GetAnim(self):
        if self.n<len(self.equAnim)-1:
            self.n+=1
        else:
            self.n=0
        return self.equAnim[self.n]
    def __iter__(self):
        self.anim=-1
    def __next__(self):
        if self.anim<len(self.KnightAnim):
            self.anim=self.anim+1
            return self.equAnim[self.anim]
        else:
            return None





from Script.Layer.MixedLayer import MixedLayer
from Script.assist.Array import Array
from Script.Player.Knight import Knight
import _thread
import pygame
from pygame.locals import *
class LevelBase:
    def __init__(self,screen,cols,rows):      
        self.cols=cols
        self.rows=rows
        self.screen=screen
        self.mixedLayer=MixedLayer(cols,rows)
    def CreateWater(self,array):
        self.mixedLayer.CreateWater(array)
        print('添加河流')
    def CreatePine(self,array):
        self.mixedLayer.CreatePine(array)
        print('添加树木')
    def CreateStone(self,array):
        self.mixedLayer.CreateStone(array)
        print('添加石头')
    def CreateStump(self,array):
        self.mixedLayer.CreateStump(array)
        print('添加树墩')
    def CreateCastle(self,array):
        self.mixedLayer.CreateCastle(array)
        print('添加城堡')
    def CreatePlayer(self,array):
        self.mixedLayer.CreatePlayer(array)
        print('添加人物')
    def CreateEquipage(self,array):
        self.mixedLayer.CreateEquipage(array)
        print('添加道具')
    def CreateBridge(self,array):
        self.mixedLayer.CreateBridge(array)
        print('添加桥')
    def CreateFlovers(self,array):
        self.mixedLayer.CreateFlovers(array)
        print('添加花朵')
    def CreateMill(self,array):
        self.mixedLayer.CreateMill(array)
        print('添加风车触发器')
    def CreateSpikes(self,array):
        self.mixedLayer.CreateSpikes(array)
        print('添加地刺')
    def CreateBoulder(self,array):
        self.mixedLayer.CreateBoulder(array)
        print('添加结界石')
    def UpdateCollision(self):
        self.mixedLayer.UpdateCollision()
        print('创建碰撞体')
    def LoadMap(self,screen):
        _thread.start_new_thread(self.mixedLayer.StartMixed,(screen,))
        #_thread.start_new_thread(self.player.Run,(screen,))
        #self.mixedLayer.StartMixed(screen)
        #self.player.forword()
    def GetPlayer(self):
        return self.knight
            



from Maps.Level01 import Level01
from Maps.Level02 import Level02
from Maps.Level03 import Level03
from Maps.Level04 import Level04
from Maps.Level05 import Level05
from Maps.Level06 import Level06
from Maps.Level07 import Level07
from Maps.Level08 import Level08
from Maps.Level09 import Level09
from Maps.Level10 import Level10
from Script.mcgame.MCGame import MCGame

def LoadLevel01():
    return Level01(MCGame.GetInstance().screen,3,3)
def LoadLevel02():
    return Level02(MCGame.GetInstance().screen,4,5)
def LoadLevel03():
    return Level03(MCGame.GetInstance().screen,7,8)
def LoadLevel04():
    return Level04(MCGame.GetInstance().screen,9,9)
def LoadLevel05():
    return Level05(MCGame.GetInstance().screen,9,12)
def LoadLevel06():
    return Level06(MCGame.GetInstance().screen,6,7)
def LoadLevel07():
    return Level07(MCGame.GetInstance().screen,9,9)
def LoadLevel08():
    return Level08(MCGame.GetInstance().screen,9,9)
def LoadLevel09():
    return Level09(MCGame.GetInstance().screen,9,9)
def LoadLevel10():
    return Level10(MCGame.GetInstance().screen,9,9)


import pygame

from pygame.locals import *
from Script.Layer.MixedLayer import MixedLayer
from Script.assist.Array import Array
import Maps
import _thread
import time
import threading
import sys
from Script.MyScreen import MyScreen
class MCGame:
    width=0
    height=0
    _instance_lock = threading.Lock()
    def __init__(self):
        pygame.init()
        pygame.display.set_caption("骑士回城堡")
        self.screen = None
        pygame.display.set_mode((0,0),(pygame.DOUBLEBUF))   
        self.maps={}
    def LoadMap(self,map):
        self.map=map
        MCGame.width = 1000
        MCGame.height = 800
        self.screen = pygame.display.set_mode((1000 , 800), 0, 32)
        self.map.LoadLevel()
        self.map.LoadMap(self.screen)
        self.player= MixedLayer.GetInstance().GetPlayer()
        MyScreen.SetScreen(1000,800,MixedLayer.GetInstance())
    def Update(self):
        MyScreen.game = 1
        while MyScreen.game:
            for event in pygame.event.get():
                if event.type == pygame.QUIT:
                    # 接收到退出时间后退出程序
                    MyScreen.game = 0
                    break
    def Start(self):
        self.map.LoadMap(self.screen)
        #_thread.start_new_thread(self.forword,())
        self.Update()
    def forword(self):
        self.player.forword()
    def right(self):
        self.player.right()
    def left(self):
        self.player.left()
    def pickup(self):
        mixed = MixedLayer.GetInstance()
        for x in mixed.equList:
            self.player.CheckEqu()
            if x.equn == self.player.equn:
                mixed.equList.remove(x)
                del x
                print('Ok')
                break
        if self.player.CheckMill():
            self.player.ActiveMill()
        pass
    def __new__(cls, *args, **kwargs):
        if not hasattr(MCGame, "_instance"):
            with MCGame._instance_lock:
                if not hasattr(MCGame, "_instance"):
                    MCGame._instance = object.__new__(cls)  
        return MCGame._instance
    @staticmethod
    def GetInstance():
        return MCGame._instance


class MyScreen:
    Width=0
    Height=0
    Mixed = None
    cols = 0
    rows = 0
    game = 0
    def SetScreen(width,height,mix):
        MyScreen.Width=width
        MyScreen.Height=height
        MyScreen.Mixed = mix
    def SetBox(cols,rows):
        MyScreen.cols = cols
        MyScreen.rows = rows


from Script.Layer.Layer import Layer
from Resources.sprites.water.Water import Water
from Script.assist.Array import Array
import pygame
from pygame.locals import *
class WaterLayer(Layer):
    def __init__(self,cols,rows,array,grass,collider):
        Layer.__init__(self,cols,rows)
        self.array=Array()
        self.array.Set(array)
        self.element=Water()
        self.CreateLayer(grass,collider)
    def CreateLayer(self,grass,collider):
        for i in range(self.cols):
            for j in range(self.rows):
                if self.array[i,j]!=0:
                    collider[i,j]=1
                    if self.array[i,j] == 2:
                        grass.myLayer[i,j]=pygame.transform.rotate(self.element.GetElement(),180)
                    else:
                        grass.myLayer[i,j]=self.element.GetElement()


from Resources.sprites.spikes.spikes import spikes as sp
from Script.MyScreen import MyScreen
from Script.AllEvent import AllEvent
from Script.Player.Knight import Knight
class spikes:
    def __init__(self,blank,i,j):
        self.e = sp()
        self.i = i
        self.j = j
        self.blank = blank
        self.active=False
        AllEvent.AddEventA(self.SetActive)
    def GetImage(self): 
        if self.active:
            self.blank.myLayer[self.i,self.j] = self.e.GetEement(1)
        else:
            self.blank.myLayer[self.i,self.j] = self.e.GetEement(0)
    def SetActive(self):
        self.active = not self.active
        if Knight.GetInstance().collider[self.i,self.j] == 6:
            Knight.GetInstance().collider[self.i,self.j] = 0
        elif Knight.GetInstance().collider[self.i,self.j] == 0:
            Knight.GetInstance().collider[self.i,self.j] = 6
        self.GetImage()





from Resources.sprites.Player.Player import Player
from Script.assist.Array import Array
from Script.Layer.MixedLayer import *
import os
import pygame
import sys
from pygame.locals import *
import Script.assist.ThreadPool
import time
import _thread
import threading
from Script.MyScreen import MyScreen
from enum import Enum
from Script.AllEvent import AllEvent
class KnightAnimation(Enum):
    #stay
    knight_stay='knight_stay'
    knight_stay_sword_shield='knight_stay_sword_shield'
    knight_stay_helmet_sword='knight_stay_helmet_sword'
    knight_stay_shield='knight_stay_shield'
    knight_stay_sword='knight_stay_sword'
    knight_stay_helmet='knight_stay_helmet'
    knight_stay_helmet_shield='knight_stay_helmet_shield'
    knight_stay_helmet_sword_shield='knight_stay_helmet_sword_shield'
    #walk
    knight_walk='knight_walk'
    knight_walk_shield='knight_walk_shield'
    knight_walk_helmet_shield='knight_walk_helmet_shield'
    knight_walk_helmet='knight_walk_helmet'
    knight_walk_sword='knight_walk_sword'
    knight_walk_helmet_sword='knight_walk_helmet_sword'
    knight_walk_sword_shield='knight_walk_sword_shield'
    knight_walk_helmet_sword_shield='knight_walk_helmet_sword_shield'
    #death
    knight_death='knight_death'
    knight_death_helmet_sword='knight_death_helmet_sword'
    knight_death_shield='knight_death_shield'
    knight_death_sword='knight_death_sword'
    knight_death_shield_sword='knight_death_shield_sword'
    knight_death_helmet_shield='knight_death_helmet_shield'
    knight_death_helmet_shield_sword='knight_death_helmet_shield_sword'
    
class Knight:
    _instance_lock = threading.Lock()
    def __init__(self,array,speed=1):
        self.speed=speed
        self.cols=array.cols
        self.rows=array.rows
        self.array=array
        self.direction=0
        temp=self.GetPos()
        self.x=temp[0]
        self.y=temp[1]
        self.posY=temp[2]
        self.posX=temp[3]
        self.player=Player()
        self.n=-1
        self.height=80
        self.width=60
        self.isWin=False
        self.isRot=False
        self.collider=Array(array.cols,array.rows)
        self.equn=0
        self.animn=0
        self.a=True
        self.b=True
        self.c=True
        self.KnightAnim=self.player.GetEement(KnightAnimation.knight_stay.value)
    def GetPos(self):
        for i in range(0,self.cols):  
             for j in range(self.rows):
                if self.array[i,j]:  
                    a=int((800-65*self.cols)/2-25 + i*65)
                    b=int((1000-65*self.rows)/2+10 + j*65)
                    #print('abij:'+str((a,b,i,j)))
                    return (b,a,i,j)
    def SetCollision(self,array):
        self.collider.Set(array)
    def GetKnightAnim(self):
        if self.n<len(self.KnightAnim)-1:
            self.n+=1
        else:
            self.n=0
        if (1-self.isWin):
            if self.isRot:
                return pygame.transform.flip(self.KnightAnim[self.n],True,False)
            else:
                return self.KnightAnim[self.n]
        else:
            if self.height==0:
                os.system("pause")
            if self.width>35:
                self.width*=0.9
                self.height*=0.9
            else:
                self.width=self.height=0
                print('过关')
            if self.isRot:
                return pygame.transform.smoothscale(pygame.transform.flip(self.KnightAnim[self.n],True,False),(int(self.width),int(self.height)))
            else:
                return pygame.transform.smoothscale(self.KnightAnim[self.n],(int(self.width),int(self.height)))
            
    def __iter__(self):
        self.anim=-1
    def __next__(self):
        if self.anim<len(self.KnightAnim):
            self.anim=self.anim+1
            return self.KnightAnim[self.anim]
        else:
            return None
    def SetXY(self,x,y):
        self.x=x
        self.y=y
    def UpdateKnightAnimation(action):
        self.KnightAnim=self.player.GetEement(action)
    def CheckCollision(self):
        if self.direction==0 and (self.collider[self.posY,self.posX+1]==1 or self.collider[self.posY,self.posX+1] == 7):
            print('触发碰撞，类型1')
            return False
        elif self.direction==1 and (self.collider[self.posY+1,self.posX]==1 or self.collider[self.posY+1,self.posX]==7):
            print('触发碰撞，类型1')
            return False
        elif self.direction==2 and (self.collider[self.posY,self.posX-1]==1 or self.collider[self.posY,self.posX-1]==7): 
            print('触发碰撞，类型1')
            return False
        elif self.direction==3 and (self.collider[self.posY-1,self.posX]==1 or self.collider[self.posY-1,self.posX]==7):
            print('触发碰撞，类型1')
            return False
        print('OK')
        self.CheckSpikes()
        return True
    def CheckMill(self):
        if self.direction==0 and self.collider[self.posY,self.posX+1] == 7:
            print('触发碰撞，类型7')
            return True
        elif self.direction==1 and self.collider[self.posY+1,self.posX]==7:
            print('触发碰撞，类型7')
            return True
        elif self.direction==2 and self.collider[self.posY,self.posX-1]==7:
            print('触发碰撞，类型7')
            return True
        elif self.direction==3 and self.collider[self.posY-1,self.posX]==7:
            print('触发碰撞，类型7')
            return True
        return False
    def CheckSpikes(self):
        if self.direction==0 and self.collider[self.posY,self.posX+1] == 6:
            print('触发碰撞，类型6')
            MyScreen.game = 0
        elif self.direction==1 and self.collider[self.posY+1,self.posX]==6:
            print('触发碰撞，类型6')
            MyScreen.game = 0
        elif self.direction==2 and self.collider[self.posY,self.posX-1]==6:
            print('触发碰撞，类型6')
            MyScreen.game = 0
        elif self.direction==3 and self.collider[self.posY-1,self.posX]==6:
            print('触发碰撞，类型6')
            MyScreen.game = 0
    def ActiveMill(self):
        MyScreen.Mixed.millLayer.element.SetRotate()
        AllEvent.RunA()
    def CheckLevel(self,pos):
        if self.collider[pos[0],pos[1]] == 2:
            self.isWin=True
            return True
        return False
    def CheckEqu(self):
        if self.collider[(self.posY,self.posX)] == 3:
            if self.a:
                self.equn = 3
                self.animn+=1
                print('触发碰撞，类型3')
                self.a=False
        elif self.collider[(self.posY,self.posX)] == 4:
            if self.b:
                self.equn = 4
                self.animn+=3
                print('触发碰撞，类型3')
                self.b=False
        elif self.collider[(self.posY,self.posX)] == 5:
            if self.c:
                self.equn = 5
                self.animn+=5
                print('触发碰撞，类型3')
                self.c=False
        self.SwitchEqu(True)
    def SwitchEqu(self,b):
        if b:
            if self.animn == 0:
                self.KnightAnim=self.player.GetEement(KnightAnimation.knight_stay.value)
            elif self.animn == 1:
                self.KnightAnim=self.player.GetEement(KnightAnimation.knight_stay_sword.value)
            elif self.animn == 3:
                self.KnightAnim=self.player.GetEement(KnightAnimation.knight_stay_shield.value)
            elif self.animn == 5:
                self.KnightAnim=self.player.GetEement(KnightAnimation.knight_stay_helmet.value)
            elif self.animn == 4:
                self.KnightAnim=self.player.GetEement(KnightAnimation.knight_stay_sword_shield.value)
            elif self.animn == 8:
                self.KnightAnim=self.player.GetEement(KnightAnimation.knight_stay_helmet_shield.value)
            elif self.animn == 6:
                self.KnightAnim=self.player.GetEement(KnightAnimation.knight_stay_helmet_sword.value)
            elif self.animn == 9:
                self.KnightAnim=self.player.GetEement(KnightAnimation.knight_stay_helmet_sword_shield.value)
        else:
            if self.animn == 0:
                self.KnightAnim=self.player.GetEement(KnightAnimation.knight_walk.value)
            elif self.animn == 1:
                self.KnightAnim=self.player.GetEement(KnightAnimation.knight_walk_sword.value)
            elif self.animn == 3:
                self.KnightAnim=self.player.GetEement(KnightAnimation.knight_walk_shield.value)
            elif self.animn == 5:
                self.KnightAnim=self.player.GetEement(KnightAnimation.knight_walk_helmet.value)
            elif self.animn == 4:
                self.KnightAnim=self.player.GetEement(KnightAnimation.knight_walk_sword_shield.value)
            elif self.animn == 8:
                self.KnightAnim=self.player.GetEement(KnightAnimation.knight_walk_helmet_shield.value)
            elif self.animn == 6:
                self.KnightAnim=self.player.GetEement(KnightAnimation.knight_walk_helmet_sword.value)
            elif self.animn == 9:
                self.KnightAnim=self.player.GetEement(KnightAnimation.knight_walk_helmet_sword_shield.value)
    def forword(self):
        self.SwitchEqu(False)
        if self.direction==0:
            if self.posX+1 < self.rows and self.CheckCollision():
                self.posX+=1
                for i in range(65):
                    time.sleep(0.01)
                    self.x+=self.speed 
        elif self.direction==1:
            if self.posY+1 < self.cols and self.CheckCollision():
                self.posY+=1
                for i in range(65):
                    time.sleep(0.01)
                    self.y+=self.speed
        elif self.direction==2:
            if self.posX-1 >= 0 and self.CheckCollision():
                self.posX-=1
                for i in range(65):
                    time.sleep(0.01)
                    self.x-=self.speed
        elif self.direction==3:
            if self.posY-1 >= 0 and self.CheckCollision():
                self.posY-=1
                for i in range(65):
                    time.sleep(0.01)
                    self.y-=self.speed
        self.SwitchEqu(True)
        if self.CheckLevel((self.posY,self.posX)):
            print('触发碰撞，类型2')
            return False
        return True
    def left(self):
        if (self.direction-1)<0:
            self.direction+=4
        self.direction-=1
        self.checkRAL()
        return True
    def right(self):
        if (self.direction+1)>3:
            self.direction-=4
        self.direction+=1
        self.checkRAL()
        return True
    def checkRAL(self):
        if self.direction==2:
            self.isRot=True
        elif self.direction==0:
            self.isRot=False
    @staticmethod
    def GetInstance():
        return Knight._instance
    def __new__(cls, *args, **kwargs):
        if not hasattr(Knight, "_instance"):
            with Knight._instance_lock:
                if not hasattr(Knight, "_instance"):
                    Knight._instance = object.__new__(cls)  
        return Knight._instance



__all__=['background',
         'castle',
         'equipage',
         'grass',
         'pine',
         'Player',
         'stone',
         'stump',
         'water',
         'bridge',
         'flovers',
         'mill',
         'spikes',
         'boulder'
         'sprite']


__all__=['Level01',
         'Level02',
         'Level03',
         'Level04',
         'Level05',
         'Level06',
         'Level07',
         'Level08',
         'Level09',
         'Level10',
         'LevelBase',
         'MapManager']


import copy
class Array:
    def __init__(self,cols=0,rows=0):
        self.cols=cols
        self.rows=rows
        self.array=[[0]*self.rows for i in range(self.cols)]
    def Set(self,array):
        self.array=array.array
    def Set2(self,array):
        self.array=array
    def SetSA(self,array):
        self.cols=len(array)
        self.rows=len(array[0])
        self.array=array
    def Get(self):
        return self.array
    def Clear(self):
        self.array.clear()
        self.array=[[0]*self.rows for i in range(self.cols)]
    def __setitem__(self, key, value): 
        self.array[key[0]][key[1]]=value
    def __getitem__(self, key):
        return self.array[key[0]][key[1]]
    def __delitem__(self, key):
        del self.array[key[0]][key[1]]
    def __len__(self,index):
        if index==0:
            return self.cols
        elif index==1:
            return self.rows
    def __call__(self):
        return self





from Script.mcgame.MCGameUnittest import *
import unittest

class myaction(unittest.TestCase):
    action=''
    @classmethod
    def setUpClass(self):
        self.action=unittest.TestSuite()
        print('开始输出操作队列：')
        
    def test_forword_run(self):
        self.action.addTest(mcgameunittest('test_forword_run'))

    def test_left_run(self):
        self.action.addTest(mcgameunittest('test_left_run'))

    def test_right_run(self):
        self.action.addTest(mcgameunittest('test_right_run'))

    def test_pickup_run(self):
        self.action.addTest(mcgameunittest('test_pickup_run'))
    def test_start_run(self):
        unittest.TextTestRunner().run(self.action)
    
class myscreen(unittest.TestCase):
    action=''
    @classmethod
    def setUpClass(self):
        self.action=unittest.TestSuite()
        print('开始屏幕刷新')
    def test_updateScreen_run(self):
        self.action.addTest(mcgameunittest('test_updateScreen_run'))

    def test_start_run(self):
        unittest.TextTestRunner().run(self.action)






